// STM32F103 System Library by NKJ. 


/////////// BitBanding ///////////
#define BBIO(reg,bit)   *((volatile unsigned char *)( PERIPH_BB_BASE + ((uint32_t)&reg-PERIPH_BASE)*32 + bit*4 ) )
#define BBSRAM(val,bit) *((volatile unsigned char *)( SRAM_BB_BASE + ((uint32_t)&val-SRAM_BASE)*32 + bit*4 ) )


void mdelay(volatile unsigned int d)  // about 1ms..
{
  volatile unsigned int n;
  for(;d>0;d--)
  {
     for(n=0;n<6000;n++);
  }
}


void uart_init1(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;  
  USART_InitTypeDef USART_InitStructure;
  
  //////////  GPIO INIT /////////
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);  //  clock enable..  
  
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_Init(GPIOA,&GPIO_InitStructure);   // porta  tx init..  

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA,&GPIO_InitStructure);   // porta  rx init..  
   
  
  //////////  USART INIT /////////
  USART_InitStructure.USART_BaudRate = 115200;
  USART_InitStructure.USART_WordLength = USART_WordLength_8b;
  USART_InitStructure.USART_StopBits = USART_StopBits_1;
  USART_InitStructure.USART_Parity = USART_Parity_No;
  USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
  USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
  
  USART_Init(USART1,&USART_InitStructure);  // Init..
  
  USART_Cmd(USART1,ENABLE);  // Usart enable..
}

void putch1(unsigned char ch)
{
  while( USART_GetFlagStatus(USART1,USART_FLAG_TXE) == RESET );
  USART_SendData(USART1,ch);
}

void put_string1(unsigned char *str)
{ unsigned int n;
  for(n=0;str[n]!='\0';n++) putch1(str[n]);
}

unsigned char getch1(void)
{
  while( USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == RESET );
  return (unsigned char)USART_ReceiveData(USART1);
}

unsigned short getc1(void)
{
  unsigned short ret;
  
  if( USART_GetFlagStatus(USART1, USART_FLAG_RXNE) == SET )
  {  ret = (unsigned char)USART_ReceiveData(USART1);
  }
  else 
  {  ret = 0x100;
  }
  return ret;
}

// External clock 12MHz
// SYSCLK = 72MHz
// HCLK = 72MHz
// PCLK1 = 36MHz
// PCLK2 = 72MHz
// ADCCLK = 36MHz
int rcc_init(void)  
{
  int ret;
  
  RCC_DeInit();   //Resets the RCC clock configuration to the default reset state.
  
  RCC_HSEConfig(RCC_HSE_ON);  // HSE enable..
  ret = RCC_WaitForHSEStartUp();  // Wait till HSE is ready

  if(ret == SUCCESS)
  {
    FLASH_SetLatency(FLASH_Latency_2); // Flash 2 wait state  (48MHz < SYSCLK <= 72MHz) 
    
    RCC_HCLKConfig(RCC_SYSCLK_Div1); // AHB prescaler(HCLK) : HCLK = SYSCLK / 1
    RCC_PCLK1Config(RCC_HCLK_Div2); // APB low-speed prescaler(APB1) : PCLK1 = HCLK / 2
    RCC_PCLK2Config(RCC_HCLK_Div1); // APB high-speed prescaler(APB2) : PCLK2 = HCLK / 1
    RCC_ADCCLKConfig(RCC_PCLK2_Div2); // ADC prescaler(ADCCLK) : ADCCLK = PCLK2 / 2
    
    RCC_PLLConfig(RCC_PLLSource_HSE_Div1, RCC_PLLMul_6); // PLLMUL : x6    PLLSRC : HSE x1            //....... PLLMul Max 12
    
    RCC_PLLCmd(ENABLE); // PLL enable..
    while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY)==RESET);  // Wait till PLL is ready
    
    RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK); // Select PLL as system clock source
    while(RCC_GetSYSCLKSource()!=0x08); // Wait till PLL is used as system clock source
  }
  
  return ret;
      
}

void vector_init(void)
{
  NVIC_SetVectorTable(NVIC_VectTab_FLASH,0);  // Set the Vector Table base location at FLASH_AREA(0x08000000)
}

int systick_init(void)
{
  RCC_ClocksTypeDef RCC_ClockStructure;  
  
  RCC_GetClocksFreq(&RCC_ClockStructure);
  if(SysTick_Config(RCC_ClockStructure.SYSCLK_Frequency / 500))   // SysTick Timer : 2msec interrupts,  SysTick_Config() MAX_value : 0xFFFFFF
    {  /* Capture error */
      return FALSE;
    }
    else
    {
      return TRUE;
    }
}
